#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "arena4xi.py">
#<abstract4xi abstract = "provides class Arena4Xi"/>
#<refer4xi reference = "heap.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Arena4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import bisect
import itertools
import mmap
import os
import sys
import threading
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    print("arena4xi.py: *** failed in import _multiprocessing ***")
try:                             # Source ones
    from multiprocessing import heap
except:
    print("arena4xi.py: *** failed in from multiprocessing import heap ***")
try:                             # Source ones
    from multiprocessing.heap import Arena
except:
    print("arena4xi.py: *** failed in from multiprocessing.heap import Arena ***")   
#"
#/>
 
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from assert_spawning4xi import assert_spawning4xi
from constant_system4xi import get_windows_system4xi 
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>



# -------------------------------------------------------------------------------------------------
# system platform
# -------------------------------------------------------------------------------------------------
sys4os = sys.platform
sys4xi = get_windows_system4xi()





if sys.platform == get_windows_system4xi():

    # =================================================================================================
    #<class4xi class = "Arena4Xi" base = "object" child = "None" synopsis = "Arena for pool and heap" usage = "
    # pool.py                            
    #     _malloc(self, size):
    #         arena = Arena(length); self._arenas.append(arena); return (arena, 0, length)
    #
    # heap.py:
    #     arena = Arena(length)
    # heap4xi.py:
    #     arena2xi = Arena4Xi(length2xi)     
    #"
    #> 
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting"/>
    # =================================================================================================
    #

    class Arena4Xi(object):
        '''
        Arena4Xi class: arena for pool and heap
        '''
      
        # ---------------------------------------------------------------------------------------------
        # Public API of this class
        # ---------------------------------------------------------------------------------------------    
        #<expose4xi exposure = "
        _exposed_ = ('sense', 'reason', 'ace')          
        #"
        #/>

        #                                        
        # ---------------------------------------------------------------------------------------------
        # sharable class variables
        # ---------------------------------------------------------------------------------------------      
        #<share4xi share = " 
        _counter = itertools.count()
        #"
        #/>
        

        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "init-of-Arena4Xi" usage = "initializes Arena4Xi">                             
        # ---------------------------------------------------------------------------------------------
        def __init__(self, size):
            '''
            initializes Arena4Xi  
            '''
            # -------------------------------------------------------------------------------------    
            # (Step 1/7)
            # -------------------------------------------------------------------------------------   
                        
            class2print = 'Arena4Xi'    
            method2print = '__init__'   
            
            message2print = "{(Step 1/7a) started" + "; init-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            # 02:58:24 01/22/13 Pacific Standard Time
    
            message2print = "{(Step 1/7b) borning" + "; arena2xi-of-Arena4Xi-borning: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 02:58:24 01/22/13 Pacific Standard Time
            
            message2print = "{(Step 1/7c) inheriting" + "; object-inheriting-by-arena2xi-of-Arena4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>
            
            message2print = "{(Step 1/7d) sharing" + "; counter-sharing-by-arena2xi-of-Arena4Xi: " + str(Arena4Xi._counter)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # count(0)
                        
            message2print = "{(Step 1/7e) passed" + "; size-passed-to-init-of-Arena4Xi: " + str(size) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 256  
            message2print = "{(Step 1/7f) commenting" + "; platform-commenting-by-init-of-Arena4Xi: Windows" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # Windows; time: 02:58:24 01/22/13 Pacific Standard Time
            
            
            # -------------------------------------------------------------------------------------    
            # (Step 2/7) OK
            # -------------------------------------------------------------------------------------                 
            try:
                self.size = size
                
                message2print = "{(Step 2/7a) set" + "; arena2xi-size-set-by-init-of-Arena4Xi to: " + str(self.size)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
                # 256
                
            except:
                message2print = "{(Step 2/7b) yielded" + "; exception-yielded-by-init-of-Arena4Xi: *** error in setting self.size ***" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                pass         
            
            # -------------------------------------------------------------------------------------    
            # (Step 3/7): OK
            # -------------------------------------------------------------------------------------                     
            try:
            
                self.name = 'pym-%d-%d' % (os.getpid(), next(Arena4Xi._counter))
            
                message2print = "{(Step 3/7a) set" + "; arena2xi-name-set-by-init-of-Arena4Xi to: " + str(self.name)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # pym-3616-0
            except:
                message2print = "{(Step 3/7b) yielded" + "; exception-yielded-by-init-of-Arena4Xi: *** error in setting self.name ***" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
                pass 
            
            # -------------------------------------------------------------------------------------    
            # (Step 4/7): OK
            # -------------------------------------------------------------------------------------         
            try:

                self.buffer = mmap.mmap(-1, self.size, tagname=self.name)
            
                message2print = "{(Step 4/7a) set" + "; arena2xi-buffer-set-by-init-of-Arena4Xi to: " + str(self.buffer)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
                # <mmap.mmap object at 0x00000000036CCC90>
                
                 
            except:         
                message2print = "{(Step 4/7b) yielded" + "; exception-yielded-by-init-of-Arena4Xi: *** error in setting self.buffer *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                pass       
                 
            # -------------------------------------------------------------------------------------    
            # (Step 5/7): to be solved exception
            # -------------------------------------------------------------------------------------         
            try:
 
            
                error2xi = _winapi.GetLastError()
                
                message2print = "{(Step 5/7a) got" + "; error2xi-got-by-init-of-Arena4Xi: " + str(error2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 0
                
                if (_winapi.GetLastError() == 0):  # dragonized
            
                    message2print = "{(Step 5/7b) asserted" + "; last-ok-asserted-by-init-of-Arena4Xi: " + str(0) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 0       
                else:
                    message2print = "{(Step 5/7c) warned" + "; tagname-warned-by-init-of-Arena4Xi: ** tagname already in use ** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)               
                
            except:
                message2print = "{(Step 5/7d) hit" + "; exception-hit-by-init-of-Arena4Xi: *** exception calling _winapi.GetLastError ***" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
                pass
                # *** error when called _winapi.GetLastError ***             

             
            # -------------------------------------------------------------------------------------    
            # (Step 6/7): OK
            # -------------------------------------------------------------------------------------                     
            try:
    
                
                self._state = (self.size, self.name)

                message2print = "{(Step 6/7a) set" + "; arena2xi-state-set-by-init-of-Arena4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                   
                # (256, 'pym-3616-0')
                 
            except:
                message2print = "{(Step 6/7b) hit" + "; exception-hit-by-init-of-Arena4Xi: *** error in setting self._state *** " + str(self._state) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                        

                pass
            
            # -------------------------------------------------------------------------------------    
            # (Step 7/7):
            # ------------------------------------------------------------------------------------- 
            
            message2print = "{(Step 7/7a) inherited" + "; object-inherited-by-arena2xi-of-Arena4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>
              
            message2print = "{(Step 7/7b) activated" + "; arena2xi-of-Arena4Xi-activated: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 02:58:24 01/22/13 Pacific Standard Time
            
            message2print = "{(Step 7/7c) shared" + "; counter-shared-by-arena2xi-of-Arena4Xi: " + str(Arena4Xi._counter)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # count(1)      
            
            message2print = "{(Step 7/7d) returning" + "; init-of-Arena4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                  
            # 02:58:24 01/22/13 Pacific Standard Time
            
        #</methodize4xi> 
                                
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "getstate-of-Arena4Xi" usage = "gets state"> 
        # ---------------------------------------------------------------------------------------------
        def __getstate__(self):
            '''
            gets state 
            '''        
    
            # -------------------------------------------------------------------------------------    
            # (Step 1/3)
            # -------------------------------------------------------------------------------------  
                       
            class2print = 'Arena4Xi'
            method2print = __getstate__

                    
            message2print = "{(Step 1/3) started" + "; getstate-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            
               
            
            # -------------------------------------------------------------------------------------    
            # (Step 2/3)
            # -------------------------------------------------------------------------------------  
            message2print = "{(Step 2/3a) passing" + "; arena2xi-passing-to-assert-spawning4xi-by-getstate-of-Arena4Xi: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            assert_spawning4xi(self)
            
            message2print = "{(Step 2/3b) performed" + "; assert-spawning4xi-performed-by-getstate-of-Arena4Xi: " + str(assert_spawning4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            # -------------------------------------------------------------------------------------    
            # (Step 3/3)
            # -------------------------------------------------------------------------------------  
                            
            message2print = "{(Step 3/3) returning" + "; state-of-arena2xi-returning-by-getstate-of-Arena4Xi: " + str(self._state) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                     
          
            return self._state
        #</methodize4xi> 
             
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "setstate-of-Arena4Xi" usage = "sets state">                             
        # ---------------------------------------------------------------------------------------------
        def __setstate__(self, state):
            '''
            sets state 
            '''
            
            # -------------------------------------------------------------------------------------    
            # (Step 1/3)
            # ------------------------------------------------------------------------------------- 
                        
            class2print = 'Arena4Xi'
            method2print = __setstate__
            

            message2print = "{(Step 1/3a) started" + "; setstate-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                

            message2print = "{(Step 1/3b) passed; state-passed-to-setstate-of-Arena4Xi: " + str(state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            
            # -------------------------------------------------------------------------------------    
            # (Step 2/3)
            # ------------------------------------------------------------------------------------- 
            
            self.size, self.name = self._state = state
                

            message2print = "{(Step 2/3a) set" + "; arena2xi-size-set-by-setstate-of-Arena4Xi to: " + str(self.size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
               
 
            message2print = "{(Step 2/3b) set" + "; arena2xi-name-set-by-setstate-of-Arena4Xi to: " + str(self.name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
              
            
            self.buffer = mmap.mmap(-1, self.size, tagname=self.name)
            
            message2print = "{(Step 2/3c) set arena2xi-buffer-set-by-setstate-of-Arena4Xi to: " + str(self.buffer)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            
            if (_winapi.GetLastError() == _winapi.ERROR_ALREADY_EXISTS):
                pass
            else:
                message2print = "{(Step 2/3d) yielded" + "; exception-yielded-by-init-of-Arena4Xi: ** exception calling _winapi.GetLastError **" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
            # -------------------------------------------------------------------------------------    
            # (Step 3/3)
            # -------------------------------------------------------------------------------------     
            message2print = "{ (Step 3/3) returning" + "; setstate-of-Arena4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)          
    
        #</methodize4xi> 
 
                                       
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "sense-of-Arena4Xi" usage = "senses Arena4Xi class objects"> 
        # ---------------------------------------------------------------------------------------------
        def sense(self):
            '''
            senses Arena4Xi class objects
            '''
              
            
            class2print = 'Arena4Xi'
            method2print = 'sense'
            
            
            message2print = "{started" + "; sense-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            # 02:58:24 01/22/13 Pacific Standard Time
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------  
            message2print = "{sensing" + "; Arena4Xi-sensing-by-sense-of-Arena4Xi: " + str(Arena4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <class 'arena4xi.Arena4Xi'>
            
            # -----------------------------------------------------------------------------------------
            # get sensed knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb2xi = sense_object4xi(self)
            
            message2print = "{got" + "; kb2xi-got-by-sense-of-Arena4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'object': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>}
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb2xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------         
            try:
                kb2xi['inheritance-of-arena2xi'] = "object-inheriting-by-arena2xi-of-Arena4Xi"
            
            except:
                message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Arena4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

          
            message2print = "{sensed" + "; Arena4Xi-sensed-by-sense-of-Arena4Xi: " + str(Arena4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # <class 'arena4xi.Arena4Xi'>
            
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------         
            message2print = "{returning" + "; kb2xi-returning-by-sense-of-Arena4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            # {'object': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>, 
            #  'inheritance-of-arena2xi': 'object-inheriting-by-arena2xi-of-Arena4Xi'}
              
            return kb2xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
        #</methodize4xi> 
     
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "reason-of-Arena4Xi" usage = "reasons Arena4Xi class objects"> 
        # ---------------------------------------------------------------------------------------------
        def reason(self):
            '''
            reasons Arena4Xi class objects
            '''
            class2print = 'Arena4Xi'
            method2print = 'reason'
    
            message2print = "{started" + "; reason-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            # 02:58:24 01/22/13 Pacific Standard Time     
   
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{reasoning" + "; Arena4Xi-reasoning-by-reason-of-Arena4Xi: "  + str(Arena4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <class 'arena4xi.Arena4Xi'>
            
            # -----------------------------------------------------------------------------------------
            # get reasoned knowledge for this object
            # -----------------------------------------------------------------------------------------         
            kb4xi = reason_object4xi(self)
            
            
            message2print = "{got" + "; kb4xi-got-by-reason-of-Arena4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'object-reason-object4xi': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>}
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb4xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------          
            message2print = "{reasoned" + "; Arena4Xi-reasone-by-reason-of-Arena4Xi: "  + str(Arena4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'arena4xi.Arena4Xi'>
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
            message2print = "{returning" + "; kb4xi-returning-by-reason-of-Arena4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            # {'object-reason-object4xi': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>}
            return kb4xi
           
        #</methodize4xi> 
     
        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "ace-of-Arena4Xi" usage = "aces Arena4Xi class objects"> 
        # ---------------------------------------------------------------------------------------------
        def ace(self):
            '''
            aces Arena4Xi class objects 
            '''
            class2print = 'Arena4Xi'
            method2print = 'ace'
            
            message2print = "{started" + "; ace-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            # 02:58:24 01/22/13 Pacific Standard Time 
    
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{acing" + "; Arena4Xi-acing-by-ace-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            # Arena4Xi
            
            # -----------------------------------------------------------------------------------------
            # get aced knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb8xi = ace_object4xi(self)  
            
            message2print = "{got" + "; kb8xi-got-by-ace-of-Arena4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'object8xi': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>}
    
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb8xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------              
            
            message2print = "{aced" + "; Arena4Xi-aced-by-ace-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # Arena4Xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
            message2print = "{returning" + "; kb8xi-returning-by-ace-of-Arena4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            # {'object8xi': <arena4xi.Arena4Xi object at 0x0000000002A6D9B0>}
            return kb8xi
                     
        #</methodize4xi> 
        
    #</class4xi>

        
else:
    # =================================================================================================
    #<class4xi class = "Arena4Xi" base = "object" child = "None" synopsis = "Arena for pool and heap" usage = "
    # pool.py                            
    #     _malloc(self, size):
    #         arena = Arena(length); self._arenas.append(arena); return (arena, 0, length)
    # heap.py:
    #     arena = Arena(length)
    # heap4xi.py:
    #     arena2xi = Arena4Xi(length2xi)     
    #"
    #> 
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # =================================================================================================
    class Arena4Xi(object):
        # ---------------------------------------------------------------------------------------------
        # Public API of this class
        # ---------------------------------------------------------------------------------------------    
        #<expose4xi expose = "
        _exposed_ = ('sense', 'reason', 'ace')          
        #"
        #/>
                                       
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "init-of-Arena4Xi" usage = "initializes Arena4Xi">                             
        # -----------------------------------------------------------------------------------------
        def __init__(self, size):
            '''
            initializes Arena4Xi
            '''
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------           
            class2print = 'Arena4Xi'
            method2print = '__init__'
    
            
            
            message2print = "{started" + "; init-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
    
            message2print = "{borning" + "; arena2xi-of-Arena4Xi-borning: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            
            message2print = "{inheriting" + "; object-inheriting-by-arena2xi-of-Arena4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   

            message2print = "{passed" + "; size-passed-to-init-of-Arena4Xi: " + str(size) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   

            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------             
            self.buffer = mmap.mmap(-1, size)
        
            message2print = "{set" + "; arena2xi-buffer-set-by-init-of-Arena4Xi to: " + str(self.buffer)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------               
            self.size = size
            
            message2print = "{set" + "; arena2xi-size-set-by-init-of-Arena4Xi to: " + str(self.size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------                
            self.name = None
            
            message2print = "{set" + "; arena2xi-name-set-by-init-of-Arena4Xi to: " + str(self.name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
               
            # -------------------------------------------------------------------------------------    
            #
            # ------------------------------------------------------------------------------------- 
            
            message2print = "{inherited" + "; object-inherited-by-arena2xi-of-Arena4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
              
            message2print = "{activated" + "; arena2xi-of-Arena4Xi-activated: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
             
                     
            message2print = "{returning" + "; init-of-Arena4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                 

        #</methodize4xi> 
    
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "sense-of-Arena4Xi" usage = "senses Arena4Xi class objects">  
        # ---------------------------------------------------------------------------------------------
        def sense(self):
            '''
            senses Arena4Xi class objects
            '''
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------                
            
            class2print = 'Arena4Xi'
            method2print = 'sense'
            
            
            message2print = "{started" + "; sense-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
                

            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------  
            message2print = "{sensing" + "; Arena4Xi-sensing-by-sense-of-Arena4Xi: " + str(Arena4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            
            # -----------------------------------------------------------------------------------------
            # get sensed knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb2xi = sense_object4xi(self)
            
            message2print = "{got" + "; kb2xi-got-by-sense-of-Arena4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb2xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------         
            try:
                kb2xi['inheritance'] = "object-inheriting-by-arena2xi-of-Arena4Xi"
                kb2xi['counter'] = Arena4Xi._counter
            except:
                pass
                            
            message2print = "{sensed" + "; Arena4Xi-sensed-by-sense-of-Arena4Xi: " + str(Arena4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------         
            message2print = "{returning" + "; kb2xi-returning-by-sense-of-Arena4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                  
              
            return kb2xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        

        #</methodize4xi> 
       
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "reason-of-Arena4Xi" usage = "reasons Arena4Xi class objects">  
        # ---------------------------------------------------------------------------------------------
        def reason(self):
            '''
            reasons Arena4Xi class objects
            '''
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------                     
            
            class2print = 'Arena4Xi'
            method2print = 'reason'
    
            
            
            message2print = "{started" + "; reason-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
               
          
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{reasoning" + "; Arena4Xi-reasoning-by-reason-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            # -----------------------------------------------------------------------------------------
            # get reasoned knowledge for this object
            # -----------------------------------------------------------------------------------------         
            kb4xi = reason_object4xi(self)
            
            
            message2print = "{got" + "; kb4xi-got-by-reason-of-Arena4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb4xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------          
            
            
              
            
            message2print = "{reasoned" + "; Arena4Xi-reasoned-by-reason-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
                        
            message2print = "{returning" + "; kb4xi-returning-by-reason-of-Arena4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                
            
            return kb4xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------            

        #</methodize4xi> 
       
        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "ace-of-Arena4Xi" usage = "aces Arena4Xi class objects">  
        # ---------------------------------------------------------------------------------------------
        def ace(self):
            '''
            aces Arena4Xi class objects 
            '''
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            class2print = 'Arena4Xi'
            method2print = 'ace'
            
            
            
            message2print = "{started" + "; ace-of-Arena4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
             
           
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{acing" + "; Arena4Xi-aced-by-ace-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            # -----------------------------------------------------------------------------------------
            # get aced knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb8xi = ace_object4xi(self)  
            
            message2print = "{got" + "; kb8xi-got-by-ace-of-Arena4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
    
    
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # format example: kb8xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------              
            message2print = "{aced" + "; Arena4Xi-aced-by-ace-of-Arena4Xi: " + "Arena4Xi" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
      
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
                         
            message2print = "{returning" + "; kb8xi-returning-by-ace-of-Arena4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            
            return kb8xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------            

        #</methodize4xi> 
    #</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_arena4xi.txt
#   sense-test-arena4xi.xml
#   reason-test-arena4xi.xml
#   ace-test-arena4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    


#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of arena4xi.py                            
# ------------------------------------------------------------------------------------------------- 
#"
#/>
